#include "GML_formula.h"

bdd GML_Formula::modal(bdd *b, int n, int m){
	return modal_var(b, n);
}
 
void GML_Formula::load_linear_program(glp_prob* problem, Premise<int>& prem){
	int n = prem.get_n();
	int m = prem.get_m();
	int no_of_valuations = prem.get_total_valuations();
	
	//glp_set_obj_dir(sc, GLP_MIN);
	glp_add_rows(problem, no_of_valuations+1);
	glp_add_cols(problem, n+m);
	
	for(int i = 1; i < no_of_valuations+1; i++)
		glp_set_row_bnds(problem, i, GLP_UP, 0.0, 0.0); // <= 0
		
	// Set row bounds for side condition
	glp_set_row_bnds(problem, no_of_valuations+1, GLP_LO, 1.0, 0.0); // >= 1
	
	for(int i=1; i < (n+m+1); i++){
		glp_set_col_kind(problem, i, GLP_IV); //integer variables
		glp_set_col_bnds(problem, i, GLP_LO, 0, 0); // r_i, s_i >= 0
	}
	
	// Arrays for inputting into glpk
	const int size = (no_of_valuations+1)*(n+m);
	double ar[size+1];
	int ia[size+1];
	int ja[size+1];
	
	// Recall GLKP takes arrays with 1 - size not 0 - size-1.
	for(int i=0; i < size; i++){
   		ia[i+1] = (i / (n+m)) + 1;
   		ja[i+1] = (i % (n+m)) + 1;	
   	}
    
 	int counter=1;
	
	// Load all valuations
	for(int i=0; i < no_of_valuations; i++){
		int temp = i;
		for(int j=0; j < n; j++){
			ar[counter] = temp & 1;
			temp >>= 1;
			counter++;
		}
		for(int j=0; j < m; j++){
			ar[counter] = -(temp & 1);
			temp >>= 1;
			counter++;
		}
	}
	
	// Load side condition at the bottom
	for(int i=0; i < n; i++){
		ar[counter]= prem.get_a_i(i)+1;
		counter++;
	}
	
	for(int j=0; j < m; j++){
		ar[counter]= -(prem.get_b_i(j));
		counter++;
	}
	
	/*
	for(int i=1; i < size+1; i++)
		cout << ia[i] << " ";
	cout << endl;
	for(int i=1; i < size+1; i++)
		cout << ja[i] << " ";
	cout << endl << endl;	
	for(int i=1; i < size+1; i++)
		cout << ar[i] << " ";
	cout << endl;
	*/
	
	glp_load_matrix(problem, size, ia, ja, ar);
}
