#ifndef MLF_DRIVER_HH
#define MLF_DRIVER_HH
#include <string>
#include "mlf-parser.tab.h"
#include "lex.yy.h"
#include "../formulas/formula.h"

using namespace std;


//Tell Flex the lexer's prototype:
# define YY_DECL                                        \
	yy::mlf_parser::token_type                         \
yylex (yy::mlf_parser::semantic_type* yylval,      \
		yy::mlf_parser::location_type* yylloc,      \
		mlf_driver& driver)
// ... and declare it for the parser's sake.
YY_DECL;


class mlf_driver{
	public:
		mlf_driver();
		~mlf_driver();

		// The parsed formula object will be construced here
		IFormula* formula;
		
	private:		
		char logic;
		
	public:
		char get_logic(){ return logic; };

		void set_formula_bdd(bdd b);
		void set_formula_gml(){ logic='g'; formula = new GML_Formula; };
		void set_formula_pml(){ logic='p'; formula = new PML_Formula; };

		/* Calls formula->variable and formula->modal respectively
		 requires ints to be pre-loaded into upper and lower. In the
		 case of only one integer being required we load it into n.*/
		bdd variable(int n);
		bdd modal(bdd *b, int n, int m);

		// Extract the formula object
		void extract_formula(IFormula* &ptr);		

		// Handling the scanner.
		void scan_begin ();
		void scan_end ();
		bool trace_scanning;

		// Run the parser.  Return 0 on success.
		int parse (const std::string& f);
		std::string file;
		bool trace_parsing;

		// Error handling.
		void error (const yy::location& l, const std::string& m);
		void error (const std::string& m);
};
#endif
