#ifndef PREMISE_HH
#define PREMISE_HH

#include <iostream>
#include <sstream>
#include <string>

#include <ext/hash_map>

#include "../formulas/rational.h"

using __gnu_cxx::hash;
using namespace std;

template<class T>
class Premise {
	protected:
		// Naming follows from the names of the rules - kept as one letter for succinctness.
		int n; // number of positive
		int m; // number of negative
		int total_valuations;
		T* a;  // modal indicies - positive
		T* b;  // modal indicies - negative
		
	public:
		Premise();
		Premise(int _n, int _m, T* _a, T* _b);		
		Premise(const Premise<T>& p);
		Premise<T>& operator=(const Premise<T>& p);
		~Premise();
		
		int get_n(){ return n; };
		int get_m(){ return m; };
		int get_total_valuations(){ return total_valuations; };
		T get_a_i(int i);
		T get_b_i(int i);
		
		struct Premise_hash {
			hash <const char*> charhash;
			size_t operator ()(const Premise<T>& p) const {
				stringstream ss;
				string s;
				ss << p.n << " " << p.m << " ";
				for(int i=0; i < p.n; i++)
					ss << p.a[i] << " ";
				for(int j=0; j < p.m; j++)
					ss << p.b[j] << " ";
				ss << '\0';
				s = ss.str();
				char buf[s.size()+1];
				for(unsigned int i=0; i < s.size(); i++)
					buf[i] = s[i];
				buf[s.size()]='\0';
				return charhash(buf);
			}	
		};
		
	struct Premise_equal { 	//TODO: this could be improved if we sort them as we input them
		bool operator()(const Premise<T>& p1, const Premise<T>& p2) const{
			if(p1.n!=p2.n)
				return false;
			if(p1.m!=p2.m)
				return false;
			for(int i=0; i < p1.n; i++)
				if(p1.a[i]!=p2.a[i])
					return false;
			for(int j=0; j < p1.m; j++)
				if(p1.b[j]!=p2.b[j])
					return false;
			return true;
		}
	};	
};


#endif
